# This file is part of Build Your Own Tests
#
# Copyright 2018 Vincent Ladeuil
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest


from byot import (
    assertions,
    fixtures,
    scenarii,
)


class TestApplyScenario(unittest.TestCase):

    def test_simple_scenario_applied(self):
        t = fixtures.make_case()
        clone = scenarii.apply_scenario(t, ('scenario', dict(a=1, b='c')))
        self.assertEqual('byot.fixtures.UnittestTest.test_pass(scenario)',
                         clone.id())
        self.assertEqual(1, clone.a)
        self.assertEqual('c', clone.b)


class TestApplyScenarios(unittest.TestCase):

    def test_apply_empty_scenarios(self):
        t = fixtures.make_case()
        self.assertEqual([], list(scenarii.apply_scenarios(t, [])))

    def test_apply_scenarios(self):
        t = fixtures.make_case()
        clones = list(scenarii.apply_scenarios(t,
                                               [('one', dict(x=1)),
                                                ('two', dict(x=2))]))
        assertions.assertLength(self, 2, clones)
        self.assertEqual('byot.fixtures.UnittestTest.test_pass(one)',
                         clones[0].id())
        self.assertEqual(1, clones[0].x)
        self.assertEqual('byot.fixtures.UnittestTest.test_pass(two)',
                         clones[1].id())
        self.assertEqual(2, clones[1].x)


class TestMultiplySuite(unittest.TestCase):

    def assertTestNames(self, expected, tests):

        def remove_prefix(test):
            return test.id()[len('byot.fixtures.UnittestTest.'):]

        self.assertEqual(expected, [remove_prefix(t) for t in tests])

    def test_multiply_single_test(self):
        scenarios = [('one', dict(x=1)), ('two', dict(x=2))]
        # It's a single test but it still needs to be part of a suite to fit
        # the API.
        suite = fixtures.make_suite(['pass'])
        clones = scenarii.multiply_suite(suite, scenarios)
        self.assertTestNames(['test_pass(one)', 'test_pass(two)'], clones)

    def test_multiply_suite(self):
        suite = fixtures.make_suite(['pass', 'fail', 'error'])
        scenarios = [('one', dict(x=1)), ('two', dict(x=2))]
        clones = scenarii.multiply_suite(suite, scenarios)
        self.assertTestNames(['test_pass(one)', 'test_pass(two)',
                              'test_fail(one)', 'test_fail(two)',
                              'test_error(one)', 'test_error(two)'],
                             clones)

    def test_multiply_suite_of_suite(self):
        suite = fixtures.make_suite(['pass', 'fail', 'error'])
        super_suite = unittest.TestSuite([suite])
        scenarios = [('one', dict(x=1)), ('two', dict(x=2))]
        clones = scenarii.multiply_suite(super_suite, scenarios)
        self.assertTestNames(['test_pass(one)', 'test_pass(two)',
                              'test_fail(one)', 'test_fail(two)',
                              'test_error(one)', 'test_error(two)'],
                             clones)

    def test_multiply_test_with_scenarios(self):
        t = fixtures.make_case()
        t.scenarios = [('one', dict(x=1)), ('two', dict(x=2))]
        suite = unittest.TestSuite([t])
        clones = scenarii.multiply_suite(suite)
        # We've found the test scenarios
        self.assertTestNames(['test_pass(one)', 'test_pass(two)'], clones)

    def test_multiply_suite_with_scenarios(self):
        suite = fixtures.make_suite(['pass'])
        suite.scenarios = [('one', dict(x=1)), ('two', dict(x=2))]
        clones = scenarii.multiply_suite(suite)
        # We've used the suite scenarios
        self.assertTestNames(['test_pass(one)', 'test_pass(two)'], clones)

    def test_multiply_suite_and_test_with_scenarios(self):
        t = fixtures.make_case()
        t.scenarios = [('one', dict(x=1)), ('two', dict(x=2))]
        suite = unittest.TestSuite([t])
        suite.scenarios = [('three', dict(y=1)), ('four', dict(y=2))]
        clones = scenarii.multiply_suite(suite)
        # We got the test ones taking over the suite ones
        self.assertTestNames(['test_pass(three)', 'test_pass(four)'], clones)


class TestMultiplyScenarios(unittest.TestCase):

    def test_one_scenario(self):
        scenarios = [('a', dict(b=1))]
        multiplied = scenarii.multiply_scenarios(scenarios)
        # It's a no-op
        self.assertEqual(scenarios, multiplied)

    def test_two_scenarios(self):
        scenarios_a = [('a', dict(x=1))]
        scenarios_b = [('b', dict(y=1, z=1)), ('c', dict(y=2, z=2))]
        multiplied = scenarii.multiply_scenarios(scenarios_a, scenarios_b)
        self.assertEqual([('a,b', dict(x=1, y=1, z=1)),
                          ('a,c', dict(x=1, y=2, z=2))], multiplied)

    def test_three_scenarios(self):
        scenarios_a = [('a', dict(x=1))]
        scenarios_b = [('b1', dict(y=1)), ('b2', dict(y=2))]
        scenarios_c = [('c1', dict(z=1)), ('c2', dict(z=2))]
        multiplied = scenarii.multiply_scenarios(scenarios_a, scenarios_b,
                                                 scenarios_c)
        self.assertEqual([('a,b1,c1', dict(x=1, y=1, z=1)),
                          ('a,b1,c2', {'x': 1, 'y': 1, 'z': 2}),
                          ('a,b2,c1', {'x': 1, 'y': 2, 'z': 1}),
                          ('a,b2,c2', {'x': 1, 'y': 2, 'z': 2})],
                         multiplied)



# This file is part of Build Your Own Tests
#
# Copyright 2018-23 Vincent Ladeuil
# Copyright 2013-2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import setuptools


import byot


def get_scripts():
    return ['byot-run']


setuptools.setup(
    name='byot',
    version='.'.join(str(c) for c in byot.__version__[0:3]),
    description=('Build Your Own Tests.'),
    author='Vincent Ladeuil',
    author_email='v.ladeuil+lp@free.fr',
    url='https://launchpad.net/byot',
    license='GPLv3',
    install_requires=['distro', 'pep8', 'pyflakes',
                      'python-subunit', 'python-testtools'],
    packages=['byot', 'byot.tests'],
    scripts=get_scripts(),
)



# This file is part of Build Your Own Tests
#
# Copyright 2018 Vincent Ladeuil
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest


from byot import matchers


class TestNameMatcher(unittest.TestCase):

    def test_defaults(self):
        nm = matchers.NameMatcher()
        self.assertFalse(nm.matches(''))
        self.assertFalse(nm.matches('foo'))

    def test_simple_include(self):
        nm = matchers.NameMatcher(includes=['foo'])
        self.assertTrue(nm.matches('foo'))
        self.assertTrue(nm.matches('XXXfooXXX'))

    def test_multiple_includes(self):
        nm = matchers.NameMatcher(includes=['foo', '^bar'])
        self.assertTrue(nm.matches('foo'))
        self.assertTrue(nm.matches('bar'))
        self.assertTrue(nm.matches('barfoo'))
        self.assertTrue(nm.matches('foobar'))
        self.assertFalse(nm.matches('bazbar'))

    def test_simple_excludes(self):
        nm = matchers.NameMatcher(includes=['.*'], excludes=['foo'])
        self.assertTrue(nm.matches('bar'))
        self.assertFalse(nm.matches('foo'))
        self.assertFalse(nm.matches('foobar'))

    def test_multiple_excludes(self):
        nm = matchers.NameMatcher(includes=['.*'], excludes=['foo$', '^bar'])
        self.assertTrue(nm.matches('baz'))
        self.assertTrue(nm.matches('footix'))
        self.assertFalse(nm.matches('foo'))
        self.assertFalse(nm.matches('barista'))



# This file is part of Build Your Own Tests
#
# Copyright 2018 Vincent Ladeuil
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from byot import matchers


class Walker(object):

    def __init__(self, root, file_matcher=None, dir_matcher=None,
                 file_handler=None, dir_handler=None,
                 sort_key=None):
        self.root = root
        if file_matcher is None:
            # Match everything
            file_matcher = matchers.NameMatcher(includes=[r'.*'])
        if dir_matcher is None:
            # Match everything
            dir_matcher = matchers.NameMatcher(includes=[r'.*'])
        self.file_matcher = file_matcher
        self.dir_matcher = dir_matcher
        self.file_handler = file_handler
        self.dir_handler = dir_handler
        self.sort_key = sort_key

    def SubWalker(self, root=None, file_matcher=None, dir_matcher=None,
                  file_handler=None, dir_handler=None,
                  sort_key=None):
        """Creates a new walker overriding walk policy.

        :param root: Optional. A new root to redirect the tree traversal.

        :param file_matcher: Optional. A new matcher to filter files.

        :param dir_matcher: Optional. A new matcher to filter directories.

        :param file_handler: A callable processing the file name. The signature
            is (walker, dir_from_root, file base name), the returned value is
            yielded by iter().

        :param dir_handler: A callable processing the dir.  The signature is
             (walker, dir_from_root), the returned values are yielded one by
             one by iter().

        :return: A new walker with the new policy, leaving the original walker
            untouched.
        """
        if root is None:
            root = self.root
        if file_matcher is None:
            file_matcher = self.file_matcher
        if dir_matcher is None:
            dir_matcher = self.dir_matcher
        if file_handler is None:
            file_handler = self.file_handler
        if dir_handler is None:
            dir_handler = self.dir_handler
        if sort_key is None:
            sort_key = self.sort_key
        # Create a clone copying the attributes that defines the current policy
        # if no specific values are provided by the caller.
        return self.__class__(root=root,
                              file_matcher=file_matcher,
                              dir_matcher=dir_matcher,
                              file_handler=file_handler,
                              dir_handler=dir_handler,
                              sort_key=sort_key)

    def join_root(self, *parts):
        """Return the walker root joined with a list of path parts.

        :param parts: Any number of path parts (including 0).

        This provides a bridge between internal needs for paths including root
        while supporting relative paths in the API. This allows the callers to
        keep clean paths from a known starting root.
        """
        return os.path.join(self.root, *parts)

    def sort_names(self, names):
        """Sort the received names."""
        if self.sort_key is None:
            return names
        return sorted(names, key=self.sort_key)

    def names(self, dir_path):
        """The list of names found in the given directory.

        :param dir_path: The path, relative to root, to list.
        """
        full_path = self.join_root(dir_path)
        if full_path == '':
            # Allowing '' to mean '.' makes for simpler code
            full_path = '.'
        return os.listdir(full_path)

    def iter(self, dir_path, names=None):
        """Iter all items in the file system from the walker's root.

        This iterates the file system starting at dir_path under root, passing
        the matching files and dirs to their respective handlers and yielding
        the results.

        :param dir_path: The path, relative to root, to iter.

        :param names: The names to iterate on in 'dir_path'. If None, all the
            existing names in 'dir_path' are iterated.
        """
        if names is None:
            names = self.sort_names(self.names(dir_path))
        for name in names:
            rel_path = os.path.join(dir_path, name)
            if self.file_handler is not None \
                    and os.path.isfile(self.join_root(rel_path)) \
                    and self.file_matcher.matches(name):
                # If the file matches, yield the result
                yield self.file_handler(self, dir_path, name)
            elif self.dir_handler is not None \
                    and os.path.isdir(self.join_root(rel_path)) \
                    and self.dir_matcher.matches(name):
                # If the dir matches, yield all results
                for name_in_dir in self.dir_handler(self, rel_path):
                    yield name_in_dir



# This file is part of Build Your Own Tests
#
# Copyright 2018 Vincent Ladeuil
# Copyright 2014, 2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest


from byot import (
    fixtures,
    walkers,
)


class TestPathFromRoot(unittest.TestCase):

    def test_just_root(self):
        w = walkers.Walker('.')
        self.assertEqual('.', w.join_root())

    def test_one_component(self):
        w = walkers.Walker('.')
        self.assertEqual('./foo', w.join_root('foo'))

    def test_several_components(self):
        w = walkers.Walker('.')
        self.assertEqual('./foo/bar', w.join_root('foo', 'bar'))

    def test_sub_walker_new_root(self):
        w = walkers.Walker('.')
        s = w.SubWalker('foo')
        # We respect the root as provided by the user (i.e. no leading '.'
        self.assertEqual('foo/bar/baz', s.join_root('bar', 'baz'))


class TestSortKey(unittest.TestCase):

    def test_no_key(self):
        w = walkers.Walker('.')
        names = ['bar', 'foo', 'baz']
        self.assertEqual(list(names), w.sort_names(list(names)))

    def test_unicode_key(self):
        w = walkers.Walker('.', sort_key=str)
        self.assertEqual(['bar', 'baz', 'foo'],
                         w.sort_names(['bar', 'foo', 'baz']))


class TestIter(unittest.TestCase):

    def setUp(self):
        super(TestIter, self).setUp()
        fixtures.set_uniq_cwd(self)
        fixtures.build_tree('''
file: foo
dir: b
file: b/a
file: b/b
dir: c
''')

    def test_do_nothing(self):
        w = walkers.Walker('.')
        self.assertEqual([], list(w.iter('.')))

    def test_get_files(self):

        def rel_path(walker, dir_path, name):
            # Return the relative path
            return os.path.join(dir_path, name)

        def recurse_for_files(walker, dir_path):
            # Recurse and yield all items
            for item in walker.iter(dir_path):
                yield item

        w = walkers.Walker('.',  sort_key=str, file_handler=rel_path,
                           dir_handler=recurse_for_files)

        self.assertEqual(
            ['./b/a', './b/b', './foo'],
            list(w.iter('.')))

    def test_get_dirs(self):

        def get_dirs(walker, dir_path):
            # Recurse and yield all items
            for item in walker.iter(dir_path):
                yield item
            # Yeild the dir itself
            yield dir_path

        w = walkers.Walker('.',  sort_key=str,
                           dir_handler=get_dirs)

        self.assertEqual(['./b', './c'], list(w.iter('.')))



# This file is part of Build Your Own Tests
#
# Copyright 2018 Vincent Ladeuil
# Copyright 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
"""Common fixtures with minimal constraints on the test class used.

A common pattern in TestCase is to add fixtures in the base class so they are
accessible to all tests. This pollutes the base classes with methods or
attributes not used by most of the tests.

Fixtures are better used via composition. But requiring an additional
'.fixtures.feature_x.attribute_y' make them harder to use.

We use a middle ground here by just joining a simpler (and a bit surprising at
first) syntax and using python's ability to handle arbitrary attributes.

Apart from that modest invasise use of test objects namespace, the only
constraint is that the test object supports the 'addCleanup' and the most usual
assertX methods. Since 'addCleanup' is provided by unittest.TestCase itself,
this is a light constraint, most python test frameworks inheriting from it.

The benefit is that these fixtures can be used on any TestCase class without
requiring inheritance which can then be used for other needs without
interferences.
"""

import inspect
import os
import shutil
import sys
import tempfile
import unittest

import testtools


_DOESNT_EXIST = object()
"""Sentinel to represent an attribute that didn't exist before patching."""


def patch(test, obj, attr_name, new):
    """Set 'obj.<attr_name>' to 'new' while the test is running.

    If 'obj' has no attribute named 'attr_name', then the patch will still go
    ahead, and the attribute will be deleted instead of restored to its
    original value at the end of the test.

    :param obj: The object to patch. Can be anything.

    :param attr_name: The attribute name on 'obj' to patch.

    :param new: The value to set 'obj.attr_name' to.
    """
    actual = getattr(obj, attr_name, _DOESNT_EXIST)
    setattr(obj, attr_name, new)

    def restore(obj, attr_name, value):
        """Restore 'obj.<attr_name>' to 'value'."""
        if value is _DOESNT_EXIST:
            delattr(obj, attr_name)
        else:
            setattr(obj, attr_name, value)
    # Using 'actual' below capture the value in the cleanup, preserving it when
    # this function (patch) is left
    test.addCleanup(restore, obj, attr_name, actual)


def set_uniq_cwd(test, prefix=None):
    """Create a temporary directory an cd into it for the test duration.

    :param test: The test to isolate in a temporary directory.

    :param prefix: A prefix to use to name the created temporary directory. It
       defaults to the test method name which help debug leaks under '/tmp'.

    This is generally called during a test setup. It requires a test providing
    the 'addCleanup' method and will set the 'uniq_dir' attribute.

    Note that it doesn't prohibit the test to write anywhere it has access to,
    but it changes its current directory so that relative paths end in a
    temporary directory. In practice, this means using simple strings paths
    instead of os.path.join'ing them which is less readable and clutter the
    test.

    This can also be used to create a home directory (as long as $HOME is also
    set appropriately) when the test needs to access '~/' files for example.

    """
    if prefix is None:
        prefix = '{}-'.format(test._testMethodName)
    test.uniq_dir = tempfile.mkdtemp(prefix=prefix, suffix='')
    test.addCleanup(shutil.rmtree, test.uniq_dir, True)
    current_dir = os.getcwd()
    test.addCleanup(os.chdir, current_dir)
    os.chdir(test.uniq_dir)


def protect_imports(test):
    """Protect sys.modules and sys.path for the test duration.

    This is useful to test imports which modifies sys.modules or requires
    modifying sys.path.
    """
    # Protect sys.modules and sys.path to be able to test imports
    patch(test, sys, 'path', list(sys.path))
    orig_modules = sys.modules.copy()

    def cleanup_modules():
        # Remove all added modules
        added = [m for m in sys.modules.keys() if m not in orig_modules]
        if added:
            for m in added:
                del sys.modules[m]
        # Restore deleted or modified modules
        sys.modules.update(orig_modules)
    test.addCleanup(cleanup_modules)


class UnittestTest(unittest.TestCase):

    # When this file is edited, the tracebacks for test_fail and test_error
    # change. To avoid spurious failures, we automate by capturing the
    # proper line.
    traceback_line = None
    # Getting the file name right is tricky, depending on whether the
    # module was just recompiled or not, __file__ can be either .py or .pyc
    # but when it appears in an exception, the .py is always used.
    traceback_file = __file__.replace('.pyc', '.py').replace('.pyo', '.py')

    def test_pass(self):
        pass

    def test_fail(self):
        self.traceback_line = inspect.currentframe().f_lineno + 1
        raise self.failureException

    def test_error(self):
        self.traceback_line = inspect.currentframe().f_lineno + 2
        raise SyntaxError('invalid syntax',
                          (self.traceback_file, 1, 1, 'No python'))

    def test_skip(self):
        self.skipTest('')

    def test_skip_reason(self):
        self.skipTest('Reason')

    @unittest.expectedFailure
    def test_expected_failure(self):
        self.traceback_line = inspect.currentframe().f_lineno + 1
        self.assertEqual(1, 0, "1 should be 0")

    @unittest.expectedFailure
    def test_unexpected_success(self):
        self.assertEqual(1, 1, "1 is not 1")


def make_case(kind='pass'):
    """Build a unittest.TestCase of a given kind.

    :param kind: The kind of test (pass, fail, see code for details). This used
        to select the method named 'test_{kind}'.

    :return: A freshly built unittest test case.

    :note: This relies on a locally defined class so test loading does not see
        it.
    """

    return UnittestTest('test_{}'.format(kind))


class TesttoolsTest(testtools.TestCase):

    # When this file is edited, the tracebacks for test_fail and test_error
    # change. To avoid spurious failures, we automate by capturing the
    # proper line.
    traceback_line = None
    traceback_file = __file__

    def test_pass(self):
        pass

    def test_fail(self):
        self.traceback_line = inspect.currentframe().f_lineno + 1
        raise self.failureException

    def test_error(self):
        self.traceback_line = inspect.currentframe().f_lineno + 1
        raise SyntaxError

    def test_skip(self):
        self.skipTest('')

    def test_skip_reason(self):
        self.skipTest('Reason')

    def test_expected_failure(self):
        # We expect the test to fail and it does
        self.expectFailure("1 should be 0", self.assertEqual, 1, 0)

    def test_unexpected_success(self):
        # We expect the test to fail but it doesn't
        self.expectFailure("1 is not 1", self.assertEqual, 1, 1)


def make_testtools_case(kind='pass'):
    """Build a testtools.TestCase of a given kind.

    :param kind: The kind of test (pass, fail, see code for details). This used
        to select the method named 'test_{kind}'.

    :return: A freshly built unittest test case.

    :note: This relies on a locally defined class so test loading does not see
        it. This differs from make_case() in subtle but invasive ways which are
        better kept in a separated function.
    """

    return TesttoolsTest('test_{}'.format(kind))


def make_suite(kinds, case_maker=None, suite_maker=None):
    """Build a test suite from a list of kinds.

    :param kinds: A list of kinds to be passed to maker to make a test case.

    :param case_maker: A callable accepting a kind and returning a test case.

    :param suite_maker: A callable returning a test suite.

    :return: A test suite
    """
    if case_maker is None:
        case_maker = make_case
    if suite_maker is None:
        suite_maker = unittest.TestSuite
    suite = suite_maker()
    suite.addTests([case_maker(kind) for kind in kinds])
    return suite


def setup_for_local_imports(test):
    """A setup method allowing test to import local files.

    This protects sys.path and sys.modules while allowing importing local
    files.
    """
    set_uniq_cwd(test)
    protect_imports(test)
    sys.path.insert(0, test.uniq_dir)


def override_env_variable(name, value):
    """Modify the environment, setting or removing the env_variable.

    :param name: The environment variable to set.

    :param value: The value to set the environment to. If None, then
        the variable will be removed.

    :return: The original value of the environment variable.
    """
    orig = os.environ.get(name)
    if value is None:
        if orig is not None:
            del os.environ[name]
    else:
        os.environ[name] = value
    return orig


def override_env(test, name, new):
    """Set an environment variable, and reset it after the test.

    :param name: The environment variable name.

    :param new: The value to set the variable to. If None, the variable is
        deleted from the environment.

    :returns: The actual variable value.
    """
    value = override_env_variable(name, new)
    test.addCleanup(override_env_variable, name, value)
    return value


isolated_environ = {
    'HOME': None,
}


def isolate_from_env(test, env=None):
    """Isolate test from the environment variables.

    This is usually called in setUp for tests that needs to modify the
    environment variables and restore them after the test is run.

    :param test: A test instance

    :param env: A dict containing variable definitions to be installed. Only
        the variables present there are protected. They are initialized with
        the provided values.
    """
    if env is None:
        env = isolated_environ
    for name, value in env.items():
        override_env(test, name, value)


def build_tree(description):
    """Build a tree described in a textual form to disk.

    The textual form describes the file contents separated by file/dir names.

    'file: <file name>' on a single line starts a file description. The file
    name must be the relative path from the tree root. The following (non
    qualified) lines describe the content of the file.

    'dir: <dir name>' on a single line starts a dir description.

    'link: <link source> <link name>' on a single line describes a symlink to
    <link source> named <link name>. The source may not exist, spaces are not
    allowed.

    :param description: A text where files and directories contents is
        described in a textual form separated by file/dir names.
    """
    cur_file = None
    for line in description.splitlines():
        if line.startswith('file: '):
            # A new file begins
            if cur_file:
                cur_file.close()
            cur_file = open(line[len('file: '):], 'w')
            continue
        if line.startswith('dir:'):
            # A new dir begins
            if cur_file:
                cur_file.close()
                cur_file = None
            os.mkdir(line[len('dir: '):])
            continue
        if line.startswith('link: '):
            # We don't support spaces in names
            link_desc = line[len('link: '):]
            try:
                source, link = link_desc.split()
            except ValueError:
                raise ValueError('Invalid link description: %s' % (link_desc,))
            os.symlink(source, link)
            continue
        if cur_file is not None:  # If no file is declared, nothing is written
            # splitlines() removed the \n, let's add it again
            cur_file.write(line + '\n')
    if cur_file:
        cur_file.close()
